IMPORT util

CONSTANT MASTER_RECORDS = 10
CONSTANT DETAIL_RECORDS_MINIMUM=5
CONSTANT DETAIL_RECORDS_MAXIMUM=100

TYPE masterRecordType RECORD
   id INTEGER,
   code CHAR(10),
   date DATE,
   desc CHAR(40)
END RECORD

TYPE detailRecordType RECORD
   id INTEGER,
   line INTEGER,
   code CHAR(10),
   desc CHAR(40),
   qty DECIMAL(11,2),
   price DECIMAL(11,2),
   value DECIMAL(11,2)
END RECORD

TYPE dataType DYNAMIC ARRAY OF RECORD
   master masterRecordType,
   detailLines DYNAMIC ARRAY OF detailRecordType
END RECORD

MAIN
DEFINE data dataType
DEFINE grw om.SaxDocumentHandler
DEFINE i,j INTEGER

   CONNECT TO "stores_demo"
   CALL generateRandomData(data)

   IF NOT fgl_report_loadCurrentSettings("masterdetailreport.4rp") THEN
      -- check report file exists
      EXIT PROGRAM 1
   END IF
   CALL fgl_report_selectPreview(TRUE)
   CALL fgl_report_selectDevice("SVG")
   LET grw = fgl_report_commitCurrentSettings()
   START REPORT masterDetailReport TO XML HANDLER grw
   FOR i = 1 TO data.getLength()
      FOR j = 1 TO data[i].detailLines.getLength()
         OUTPUT TO REPORT masterDetailReport(data[i].master.*, data[i].detailLines[j].*)
      END FOR
   END FOR
   FINISH REPORT masterDetailReport
END MAIN

REPORT masterDetailReport(master, detail)
DEFINE master masterRecordType
DEFINE detail detailRecordType
DEFINE subtotal RECORD
   lineCount INTEGER,
   qtyTotal DECIMAL(11,2),
   valueTotal DECIMAL(11,2)
END RECORD

   ORDER EXTERNAL BY master.id, detail.line

FORMAT
   BEFORE GROUP OF master.id
      PRINT master.*

   ON EVERY ROW
      PRINT detail.*

   AFTER GROUP OF master.id
      LET subtotal.lineCount = COUNT(*)
      LET subtotal.qtyTotal= GROUP SUM(detail.qty)
      LET subtotal.valueTotal = GROUP SUM(detail.value)
      PRINT subtotal.*
END REPORT

#! use pass by reference to populate data
FUNCTION generateRandomData(data)
DEFINE data dataType
DEFINE i,j INTEGER
DEFINE detail_count INTEGER

   FOR i = 1 TO MASTER_RECORDS
      LET data[i].master.id = i
      LET data[i].master.code = random_code(10)
      LET data[i].master.desc =  random_desc(1,4,3,10)
      LET data[i].master.date = TODAY - util.Math.rand(10)
   
      LET detail_count = DETAIL_RECORDS_MINIMUM+util.Math.rand(DETAIL_RECORDS_MAXIMUM-DETAIL_RECORDS_MINIMUM)
      FOR j = 1 TO detail_count
         LET data[i].detailLines[j].id = i
         LET data[i].detailLines[j].line = j
         LET data[i].detailLines[j].code = random_code(10)
         LET data[i].detailLines[j].desc = random_desc(1,4,3,10)
         LET data[i].detailLines[j].price = util.Math.rand(10000)/100
         LET data[i].detailLines[j].qty = util.Math.rand(100)
         LET data[i].detailLines[j].value =  data[i].detailLines[j].price * data[i].detailLines[j].qty
      END FOR
   END FOR
END FUNCTION

FUNCTION random_code(letters)
DEFINE letters SMALLINT
DEFINE i INTEGER
DEFINE sb base.StringBuffer

   LET sb = base.StringBuffer.create()
   FOR i = 1 TO letters
      CALL sb.append(ASCII(util.math.rand(26)+65))
   END FOR
   RETURN sb.toString()
END FUNCTION

FUNCTION random_desc(min_word, max_word, min_letter, max_letter)
DEFINE min_word, max_word, min_letter, max_letter SMALLINT
DEFINE words, letters, w, l SMALLINT
DEFINE sb base.StringBuffer

   LET sb = base.StringBuffer.create()
   LET words = util.Math.rand(max_Word-min_word) + min_word

   FOR w = 1 TO words
      LET letters = util.Math.rand(max_letter-min_letter) + min_letter
      IF w > 1 THEN
         CALL sb.append(" ")
      END IF
      CALL sb.append(ASCII(65+util.math.rand(26)))
      FOR l = 2 TO letters
         CALL sb.append(ASCII(97+util.Math.rand(26)))
      END FOR
   END FOR
   RETURN sb.toString()
END FUNCTION